package it.polito.bigdata.spark;

import org.apache.spark.api.java.*;
import org.apache.spark.SparkConf;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import scala.Tuple2;

public class SparkDriver {

	public static void main(String[] args) {

		// The following two lines are used to switch off some verbose log messages
		Logger.getLogger("org").setLevel(Level.OFF);
		Logger.getLogger("akka").setLevel(Level.OFF);

		// String companiesPath; Useless for this program
		String dataCenterPath, dailyPwrConsPath;
		String outputPath1, outputPath2;

		// companiesPath = args[0]; Useless for this program
		dataCenterPath = args[1];
		dailyPwrConsPath = args[2];
		outputPath1 = args[3];
		outputPath2 = args[4];

		// Create a configuration object and set the name of the application
		SparkConf conf = new SparkConf().setAppName("Spark - Exam20220906")
		.setMaster("local");

		// Use the following command to create the SparkConf object if you want to run
		// your application inside Eclipse.
		// Remember to remove .setMaster("local") before running your application on the
		// cluster
		// SparkConf conf=new SparkConf().setAppName("Spark Lab #5").setMaster("local");

		// Create a Spark Context object
		JavaSparkContext sc = new JavaSparkContext(conf);

		//JavaRDD<String> companiesRDD = sc.textFile(companiesPath); Useless for this program
		JavaRDD<String> dataCenterRDD = sc.textFile(dataCenterPath).cache();
		JavaRDD<String> pwrConsRDD = sc.textFile(dailyPwrConsPath);

		// Part 1
		// Count the total number of data centers worl-wide and compute the threshold
		int threshold = (int) (dataCenterRDD.count() * 0.9);

		// Filter only the dates associated with high power consumption
		// and prepare a pairRDD with
		// key = date
		// value = +1
		// to count the number of data centers with high power consumption for each day.
		JavaPairRDD<String, Integer> highPwrConsDCPerDay = pwrConsRDD.filter(line -> {
			String[] fields = line.split(",");
			double pwrCons = Double.parseDouble(fields[2]);
			return pwrCons >= 1000;
		}).mapToPair(line -> {
			String[] fields = line.split(",");
			String date = fields[1];
			return new Tuple2<>(date, 1);
		}).reduceByKey((v1, v2) -> v1 + v2);

		JavaRDD<String> res1 = highPwrConsDCPerDay.filter(t -> t._2() >= threshold).keys();

		res1.saveAsTextFile(outputPath1);

		// Part 2

		// Consider the power consumptions and keep only the entries related to year 2021
		// and obtain the following pairRDD
		// key = codDC
		// value = kWh
		// and use a reduceByKey to sum the power consumption for the entire year for
		// each data center
		JavaPairRDD<String, Double> yearlyPwrCons = pwrConsRDD.filter(line -> {
			String[] fields = line.split(",");
			return fields[1].startsWith("2021");
		}).mapToPair(line -> {
			String[] fields = line.split(",");
			String codDC = fields[0];
			double pwrCons = Double.parseDouble(fields[2]);
			return new Tuple2<>(codDC, pwrCons);
		}).reduceByKey((v1, v2) -> v1 + v2);

		// for each data center, keep the continent information
		// key = codDC
		// value = continent
		JavaPairRDD<String, String> dcAndContinent = dataCenterRDD.mapToPair(line -> {
			String[] fields = line.split(",");
			String codDC = fields[0];
			String continent = fields[4];
			return new Tuple2<>(codDC, continent);
		});

		// Join yearlyPwrCons with dcAndContinent and
		// returns pairs
		// key = continent
		// value = (+1, kWhPerDataCenter2021)
		JavaPairRDD<String, Tuple2<Integer, Double>> continentOnePwr = yearlyPwrCons.join(dcAndContinent)
				.mapToPair(t -> new Tuple2<String, Tuple2<Integer, Double>>(t._2()._2(), new Tuple2<Integer, Double>(1, t._2()._1())));

		// Sum the value parts to compute for each continent
		// the number of data centers and the total power consumption in the year 2021.
		// Finally, compute the avg power consumption
		// key = continent
		// value = (the number of data centers, avg power consumption in the year 2021)
		JavaPairRDD<String, Tuple2<Integer, Double>> numDCandAvgPwrCons = continentOnePwr
				.reduceByKey((t1, t2) -> new Tuple2<Integer, Double>(t1._1() + t2._1(), t1._2() + t2._2()))
				.mapValues(t -> new Tuple2<Integer, Double>(t._1(), t._2() / t._1())).cache();

		// compute the maximum number of data centers and the maximum avg consumption in
		// the 2021 among the continents
		Tuple2<Integer, Double> maxDCAndConsumptionPerContinentThresholds = numDCandAvgPwrCons.values()
				.reduce((t1, t2) -> new Tuple2<Integer, Double>(t1._1() > t2._1() ? t1._1() : t2._1(),
						t1._2() > t2._2() ? t1._2() : t2._2()));

		// filter only those continents for which both constraints are satisfied
		JavaRDD<String> res2 = numDCandAvgPwrCons
				.filter(t -> (t._2()._1().compareTo(maxDCAndConsumptionPerContinentThresholds._1())==0
						&& t._2()._2().compareTo(maxDCAndConsumptionPerContinentThresholds._2())==0))
				.keys();


		res2.saveAsTextFile(outputPath2);
		sc.close();
	}
}
